`timescale 1ns/1ps
`include "common_defs.v"

// 1-bit priority-based arbitration slice
module prior_arb_cell (pri, car, req, grt, car_next);

input pri, car, req;
output grt, car_next;

assign grt = req & (car | pri);
assign car_next = !req & (car | pri);

endmodule


// 1-bit generator of round-robin priority
module rr_cell (clk, rst, grt_prev, grt_any, pri_rst, pri);

input clk, rst, grt_prev, grt_any, pri_rst;
output reg pri;

always @ (posedge clk) begin
  if (rst)
    pri <= pri_rst;
  else
    pri <= !grt_any & pri | grt_prev;
end

endmodule


// - arbiter with 5 inputs, only one gets granted at a time
// - grant logic is combinational, but priority updates once per cycle
// - grants only if input is not empty and output is not full
// NOTICE: The arbiter has a combinational loop caused by the circular carry chain.
// The loop may prevent from performing correct timing analysis.
// In order to break the loop we instantiate two copies of the elementary
// 5x1 arbiter block and OR the grant signals from both instances.
// For more information see Dally's book on Interconnection Networks, ch. 18.4, page 354.
module arbiter_rr_5x1_block (clk, rst, empty, full, pri, req, car_in, car_out, grt);

input clk, rst, full, car_in;
input [`DIR_CNT-1:0] empty, req;
input [`DIR_CNT-1:0] pri;
output car_out;
output [`DIR_CNT-1:0] grt;

// internal signals
wire [`DIR_CNT-1:0] car; // carries
wire [`DIR_CNT-1:0] req_int; // internal request
wire [`DIR_CNT-1:0] grt_int; // internal grant

// redefine request to actually request only when 
// both input data and output storage are available;
assign req_int[`EAST_DIR] = req[`EAST_DIR] & !empty[`EAST_DIR] & !full;
assign req_int[`NORTH_DIR] = req[`NORTH_DIR] & !empty[`NORTH_DIR] & !full;
assign req_int[`WEST_DIR] = req[`WEST_DIR] & !empty[`WEST_DIR] & !full;
assign req_int[`SOUTH_DIR] = req[`SOUTH_DIR] & !empty[`SOUTH_DIR] & !full;
assign req_int[`LOCAL_DIR] = req[`LOCAL_DIR] & !empty[`LOCAL_DIR] & !full;

// use carry_in and carry_out instead of car[`EAST_DIR]
// in order to break the carry loop
prior_arb_cell pac_east(pri[`EAST_DIR], car_in, req_int[`EAST_DIR],
                        grt_int[`EAST_DIR], car[`NORTH_DIR]);
prior_arb_cell pac_north(pri[`NORTH_DIR], car[`NORTH_DIR], req_int[`NORTH_DIR],
                         grt_int[`NORTH_DIR], car[`WEST_DIR]);
prior_arb_cell pac_west(pri[`WEST_DIR], car[`WEST_DIR], req_int[`WEST_DIR],
                        grt_int[`WEST_DIR], car[`SOUTH_DIR]);
prior_arb_cell pac_south(pri[`SOUTH_DIR], car[`SOUTH_DIR], req_int[`SOUTH_DIR],
                         grt_int[`SOUTH_DIR], car[`LOCAL_DIR]);
prior_arb_cell pac_local(pri[`LOCAL_DIR], car[`LOCAL_DIR], req_int[`LOCAL_DIR],
                         grt_int[`LOCAL_DIR], car_out);

assign grt = grt_int;

endmodule


// This is the final arbiter which instantiates two
// subarbiter copies to resolve the carry chain loop.
// This is the module to be instantiated by the router.
module arbiter_rr_5x1 (clk, rst, empty, full, req, grt);

input clk, rst, full;
input [`DIR_CNT-1:0] empty, req;
output [`DIR_CNT-1:0] grt;

// internal signals
wire car_in; // input carry
wire car_arb1_to_arb2; // carry between the arbiters
wire car_out; // output carry, not used
wire [`DIR_CNT-1:0] grt_arb1; // grants from the first arbiter
wire [`DIR_CNT-1:0] grt_arb2; // grants from the second arbiter
wire [`DIR_CNT-1:0] pri; // round-robin priority
wire [`DIR_CNT-1:0] pri_rst = 1; // priority at reset
wire grt_any; // any signal is granted

assign car_in = 1'b0;

assign grt_any = |grt;

rr_cell rrc_east(clk, rst, grt[`LOCAL_DIR], grt_any,
                  pri_rst[`EAST_DIR], pri[`EAST_DIR]);
rr_cell rrc_north(clk, rst, grt[`EAST_DIR], grt_any,
                  pri_rst[`NORTH_DIR], pri[`NORTH_DIR]);
rr_cell rrc_west(clk, rst, grt[`NORTH_DIR], grt_any,
                 pri_rst[`WEST_DIR], pri[`WEST_DIR]);
rr_cell rrc_south(clk, rst, grt[`WEST_DIR], grt_any,
                  pri_rst[`SOUTH_DIR], pri[`SOUTH_DIR]);
rr_cell rrc_local(clk, rst, grt[`SOUTH_DIR], grt_any,
                  pri_rst[`LOCAL_DIR], pri[`LOCAL_DIR]);

// instantiate both arbiter blocks
arbiter_rr_5x1_block arb1(clk, rst, empty, full, pri, req, car_in, car_arb1_to_arb2, grt_arb1);
arbiter_rr_5x1_block arb2(clk, rst, empty, full, pri, req, car_arb1_to_arb2, car_out, grt_arb2);

// OR the grant signals from both arbiters
assign grt = grt_arb1 | grt_arb2;

endmodule

