`timescale 1ns/1ps

// - head flit appears at fifo output as soon as it is written (pass-through implementation)
// - upon read, the head flit gets eliminated from the fifo and next flit appears at the output
module fifo (clk, rst, write, data_in, read, data_out, empty, full);

parameter WIDTH = 2; // number of bits per flit
parameter ADDR_WIDTH = 2; // log(SIZE)
parameter SIZE = 4;  // number of flits

parameter BUF_SIZE = (1 << ADDR_WIDTH);

input clk, rst, write, read;
input [WIDTH-1:0] data_in;
output reg empty, full;
output reg [WIDTH-1:0] data_out;

reg [WIDTH-1:0] buffer [0:BUF_SIZE-1];
reg [ADDR_WIDTH-1:0] rptr, wptr;
wire [ADDR_WIDTH-1:0] rptr_inc;
reg [ADDR_WIDTH:0] cnt;

integer k;

// setup empty and full
always @ (cnt or rst) begin
  if (rst) begin
    empty <= 1;
    full <= 0;
  end
  else begin
    empty <= (cnt == 0);
    full <= (cnt == SIZE);
  end
end

assign rptr_inc = rptr + 1;

// read
always @ (posedge clk) begin
  if (rst) begin
    rptr <= 0;
    data_out <= 0;
  end
  else begin
    if (empty)
      data_out <= data_in;
    else if (write && !full && read && cnt == 1) begin
      rptr <= rptr_inc;
      data_out <= data_in;
    end
    else if (read && !empty) begin
      rptr <= rptr_inc;
      data_out <= buffer [(rptr_inc > BUF_SIZE-1) ? 0 : (rptr_inc)];
    end
    else
      data_out <= buffer [rptr];
  end
end

// write
always @ (posedge clk) begin
  if (rst) begin
    wptr <= 0;
    for (k = 0; k < BUF_SIZE; k = k+1)
      buffer[k] <= 0;
  end
  else begin
    // Never write when 'cnt' == SIZE to drop 'full' and resume data flow
    if (write && !full) begin
      buffer [wptr] <= data_in;
      wptr <= wptr + 1;
    end
  end
end

// update counter
always @ (posedge clk) begin
  if (rst) begin
    cnt <= 0;
  end
  else begin
    if (read && !empty && write && !full) cnt <= cnt;
    else if (read && !empty) cnt <= cnt - 1;
    else if (write && !full) cnt <= cnt + 1;
    else cnt <= cnt;
  end
end

endmodule

