`timescale 1ns/1ps
`include "common_defs.v"

// router handshake interface:
//   req_in - input request (write to router)
//   rdy_in - input ready (not full)
//   req_out - output request (not empty)
//   rdy_out - output ready (read from router)
module router (clk, rst, req_in, rdy_in, data_in, req_out, rdy_out, data_out);

// ==================== PARAMETER DEFINITION ====================
parameter LINKWIDTH = `IC_WIDTH;
parameter IN_PORT_CNT = `DIR_CNT; // number of input ports of the router
parameter OUT_PORT_CNT = `DIR_CNT; // number of output ports of the router

parameter TILE_X = 4'd1; // x-coordinate of the tile
parameter TILE_Y = 4'd1; // y-coordinate of the tile

// ==================== INTERFACE ====================
input clk, rst;

input [IN_PORT_CNT-1:0] req_in;
input [LINKWIDTH*IN_PORT_CNT-1:0] data_in;
output [IN_PORT_CNT-1:0] rdy_in;
output [OUT_PORT_CNT-1:0] req_out;
output [LINKWIDTH*OUT_PORT_CNT-1:0] data_out;
input [OUT_PORT_CNT-1:0] rdy_out;

// ==================== INTERNAL SIGNALS ====================
wire [IN_PORT_CNT*4-1:0] x_dst, y_dst; // tile coordinates sent to route computation
wire [IN_PORT_CNT*OUT_PORT_CNT-1:0] route_dir; // computed routes (directions)
wire [IN_PORT_CNT*OUT_PORT_CNT-1:0] req_from_in_ports; // input port requests
wire [LINKWIDTH*IN_PORT_CNT-1:0] data_in_fifo_head; // head of input fifo
wire [IN_PORT_CNT-1:0] in_fifo_empty;
wire [OUT_PORT_CNT-1:0] out_fifo_full;
// req_to_out_ports are just reordered req_from_in_ports
wire [IN_PORT_CNT*OUT_PORT_CNT-1:0] req_to_out_ports; // requests to output ports (arbiter input)
wire [IN_PORT_CNT*OUT_PORT_CNT-1:0] grt_to_out_ports; // grants to output ports (arbiter output)
wire [LINKWIDTH*OUT_PORT_CNT-1:0] data_out_fifo_tail; // tail of output fifo
wire [IN_PORT_CNT-1:0] in_port_read;
wire [OUT_PORT_CNT-1:0] out_port_write;
wire [IN_PORT_CNT-1:0] full;
wire [OUT_PORT_CNT-1:0] empty;

// ==================== MODULES ====================

/// TODO: replace multiple instances with generate, when moved from cver

// input ports
input_port #(
    .LINKWIDTH(LINKWIDTH),
    .OUT_PORT_CNT(OUT_PORT_CNT),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  i_north(
    .clk(clk),
    .rst(rst),
    .route_dir(route_dir[OUT_PORT_CNT*(`NORTH_DIR+1)-1:OUT_PORT_CNT*`NORTH_DIR]),
    .write(req_in[`NORTH_DIR]),
    .data_in(data_in[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR]),
    .read(in_port_read[`NORTH_DIR]),
    .data_out(data_in_fifo_head[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR]),
    .empty(in_fifo_empty[`NORTH_DIR]),
    .full(full[`NORTH_DIR]),
    .x_dst(x_dst[4*(`NORTH_DIR+1)-1:4*`NORTH_DIR]),
    .y_dst(y_dst[4*(`NORTH_DIR+1)-1:4*`NORTH_DIR]),
    .req(req_from_in_ports[OUT_PORT_CNT*(`NORTH_DIR+1)-1:OUT_PORT_CNT*`NORTH_DIR]));

input_port #(
    .LINKWIDTH(LINKWIDTH),
    .OUT_PORT_CNT(OUT_PORT_CNT),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  i_west(
    .clk(clk),
    .rst(rst),
    .route_dir(route_dir[OUT_PORT_CNT*(`WEST_DIR+1)-1:OUT_PORT_CNT*`WEST_DIR]),
    .write(req_in[`WEST_DIR]),
    .data_in(data_in[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR]),
    .read(in_port_read[`WEST_DIR]),
    .data_out(data_in_fifo_head[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR]),
    .empty(in_fifo_empty[`WEST_DIR]),
    .full(full[`WEST_DIR]),
    .x_dst(x_dst[4*(`WEST_DIR+1)-1:4*`WEST_DIR]),
    .y_dst(y_dst[4*(`WEST_DIR+1)-1:4*`WEST_DIR]),
    .req(req_from_in_ports[OUT_PORT_CNT*(`WEST_DIR+1)-1:OUT_PORT_CNT*`WEST_DIR]));

input_port #(
    .LINKWIDTH(LINKWIDTH),
    .OUT_PORT_CNT(OUT_PORT_CNT),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  i_south(
    .clk(clk),
    .rst(rst),
    .route_dir(route_dir[OUT_PORT_CNT*(`SOUTH_DIR+1)-1:OUT_PORT_CNT*`SOUTH_DIR]),
    .write(req_in[`SOUTH_DIR]),
    .data_in(data_in[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR]),
    .read(in_port_read[`SOUTH_DIR]),
    .data_out(data_in_fifo_head[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR]),
    .empty(in_fifo_empty[`SOUTH_DIR]),
    .full(full[`SOUTH_DIR]),
    .x_dst(x_dst[4*(`SOUTH_DIR+1)-1:4*`SOUTH_DIR]),
    .y_dst(y_dst[4*(`SOUTH_DIR+1)-1:4*`SOUTH_DIR]),
    .req(req_from_in_ports[OUT_PORT_CNT*(`SOUTH_DIR+1)-1:OUT_PORT_CNT*`SOUTH_DIR]));

input_port #(
    .LINKWIDTH(LINKWIDTH),
    .OUT_PORT_CNT(OUT_PORT_CNT),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  i_east(
    .clk(clk),
    .rst(rst),
    .route_dir(route_dir[OUT_PORT_CNT*(`EAST_DIR+1)-1:OUT_PORT_CNT*`EAST_DIR]),
    .write(req_in[`EAST_DIR]),
    .data_in(data_in[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR]),
    .read(in_port_read[`EAST_DIR]),
    .data_out(data_in_fifo_head[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR]),
    .empty(in_fifo_empty[`EAST_DIR]),
    .full(full[`EAST_DIR]),
    .x_dst(x_dst[4*(`EAST_DIR+1)-1:4*`EAST_DIR]),
    .y_dst(y_dst[4*(`EAST_DIR+1)-1:4*`EAST_DIR]),
    .req(req_from_in_ports[OUT_PORT_CNT*(`EAST_DIR+1)-1:OUT_PORT_CNT*`EAST_DIR]));

input_port #(
    .LINKWIDTH(LINKWIDTH),
    .OUT_PORT_CNT(OUT_PORT_CNT),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  i_local(
    .clk(clk),
    .rst(rst),
    .route_dir(route_dir[OUT_PORT_CNT*(`LOCAL_DIR+1)-1:OUT_PORT_CNT*`LOCAL_DIR]),
    .write(req_in[`LOCAL_DIR]),
    .data_in(data_in[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR]),
    .read(in_port_read[`LOCAL_DIR]),
    .data_out(data_in_fifo_head[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR]),
    .empty(in_fifo_empty[`LOCAL_DIR]),
    .full(full[`LOCAL_DIR]),
    .x_dst(x_dst[4*(`LOCAL_DIR+1)-1:4*`LOCAL_DIR]),
    .y_dst(y_dst[4*(`LOCAL_DIR+1)-1:4*`LOCAL_DIR]),
    .req(req_from_in_ports[OUT_PORT_CNT*(`LOCAL_DIR+1)-1:OUT_PORT_CNT*`LOCAL_DIR]));

// route computation
route_comp_1port rc_north(
    .x_dst(x_dst[4*(`NORTH_DIR+1)-1:4*`NORTH_DIR]),
    .y_dst(y_dst[4*(`NORTH_DIR+1)-1:4*`NORTH_DIR]),
    .x_cur(TILE_X[3:0]),
    .y_cur(TILE_Y[3:0]),
    .dir(route_dir[OUT_PORT_CNT*(`NORTH_DIR+1)-1:OUT_PORT_CNT*`NORTH_DIR]));

route_comp_1port rc_west(
    .x_dst(x_dst[4*(`WEST_DIR+1)-1:4*`WEST_DIR]),
    .y_dst(y_dst[4*(`WEST_DIR+1)-1:4*`WEST_DIR]),
    .x_cur(TILE_X[3:0]),
    .y_cur(TILE_Y[3:0]),
    .dir(route_dir[OUT_PORT_CNT*(`WEST_DIR+1)-1:OUT_PORT_CNT*`WEST_DIR]));

route_comp_1port rc_south(
    .x_dst(x_dst[4*(`SOUTH_DIR+1)-1:4*`SOUTH_DIR]),
    .y_dst(y_dst[4*(`SOUTH_DIR+1)-1:4*`SOUTH_DIR]),
    .x_cur(TILE_X[3:0]),
    .y_cur(TILE_Y[3:0]),
    .dir(route_dir[OUT_PORT_CNT*(`SOUTH_DIR+1)-1:OUT_PORT_CNT*`SOUTH_DIR]));

route_comp_1port rc_east(
    .x_dst(x_dst[4*(`EAST_DIR+1)-1:4*`EAST_DIR]),
    .y_dst(y_dst[4*(`EAST_DIR+1)-1:4*`EAST_DIR]),
    .x_cur(TILE_X[3:0]),
    .y_cur(TILE_Y[3:0]),
    .dir(route_dir[OUT_PORT_CNT*(`EAST_DIR+1)-1:OUT_PORT_CNT*`EAST_DIR]));

route_comp_1port rc_local(
    .x_dst(x_dst[4*(`LOCAL_DIR+1)-1:4*`LOCAL_DIR]),
    .y_dst(y_dst[4*(`LOCAL_DIR+1)-1:4*`LOCAL_DIR]),
    .x_cur(TILE_X[3:0]),
    .y_cur(TILE_Y[3:0]),
    .dir(route_dir[OUT_PORT_CNT*(`LOCAL_DIR+1)-1:OUT_PORT_CNT*`LOCAL_DIR]));

// arbiters
arbiter_rr_5x1 arb_north(
    .clk(clk),
    .rst(rst),
    .empty(in_fifo_empty),
    .full(out_fifo_full[`NORTH_DIR]),
    .req(req_to_out_ports[IN_PORT_CNT*(`NORTH_DIR+1)-1:IN_PORT_CNT*`NORTH_DIR]),
    .grt(grt_to_out_ports[IN_PORT_CNT*(`NORTH_DIR+1)-1:IN_PORT_CNT*`NORTH_DIR]));

arbiter_rr_5x1 arb_west(
    .clk(clk),
    .rst(rst),
    .empty(in_fifo_empty),
    .full(out_fifo_full[`WEST_DIR]),
    .req(req_to_out_ports[IN_PORT_CNT*(`WEST_DIR+1)-1:IN_PORT_CNT*`WEST_DIR]),
    .grt(grt_to_out_ports[IN_PORT_CNT*(`WEST_DIR+1)-1:IN_PORT_CNT*`WEST_DIR]));

arbiter_rr_5x1 arb_south(
    .clk(clk),
    .rst(rst),
    .empty(in_fifo_empty),
    .full(out_fifo_full[`SOUTH_DIR]),
    .req(req_to_out_ports[IN_PORT_CNT*(`SOUTH_DIR+1)-1:IN_PORT_CNT*`SOUTH_DIR]),
    .grt(grt_to_out_ports[IN_PORT_CNT*(`SOUTH_DIR+1)-1:IN_PORT_CNT*`SOUTH_DIR]));

arbiter_rr_5x1 arb_east(
    .clk(clk),
    .rst(rst),
    .empty(in_fifo_empty),
    .full(out_fifo_full[`EAST_DIR]),
    .req(req_to_out_ports[IN_PORT_CNT*(`EAST_DIR+1)-1:IN_PORT_CNT*`EAST_DIR]),
    .grt(grt_to_out_ports[IN_PORT_CNT*(`EAST_DIR+1)-1:IN_PORT_CNT*`EAST_DIR]));

arbiter_rr_5x1 arb_local(
    .clk(clk),
    .rst(rst),
    .empty(in_fifo_empty),
    .full(out_fifo_full[`LOCAL_DIR]),
    .req(req_to_out_ports[IN_PORT_CNT*(`LOCAL_DIR+1)-1:IN_PORT_CNT*`LOCAL_DIR]),
    .grt(grt_to_out_ports[IN_PORT_CNT*(`LOCAL_DIR+1)-1:IN_PORT_CNT*`LOCAL_DIR]));


// xbar
xbar_5x5 #(
    .XBAR_LINKWIDTH(LINKWIDTH))
  xb(
    .sel(grt_to_out_ports),
    .data_in(data_in_fifo_head),
    .data_out(data_out_fifo_tail));

// output ports
output_port #(
    .LINKWIDTH(LINKWIDTH),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  o_north(
    .clk(clk),
    .rst(rst),
    .write(out_port_write[`NORTH_DIR]),
    .data_in(data_out_fifo_tail[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR]),
    .read(rdy_out[`NORTH_DIR]),
    .data_out(data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR]),
    .empty(empty[`NORTH_DIR]),
    .full(out_fifo_full[`NORTH_DIR]));

output_port #(
    .LINKWIDTH(LINKWIDTH),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  o_west(
    .clk(clk),
    .rst(rst),
    .write(out_port_write[`WEST_DIR]),
    .data_in(data_out_fifo_tail[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR]),
    .read(rdy_out[`WEST_DIR]),
    .data_out(data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR]),
    .empty(empty[`WEST_DIR]),
    .full(out_fifo_full[`WEST_DIR]));

output_port #(
    .LINKWIDTH(LINKWIDTH),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  o_south(
    .clk(clk),
    .rst(rst),
    .write(out_port_write[`SOUTH_DIR]),
    .data_in(data_out_fifo_tail[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR]),
    .read(rdy_out[`SOUTH_DIR]),
    .data_out(data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR]),
    .empty(empty[`SOUTH_DIR]),
    .full(out_fifo_full[`SOUTH_DIR]));

output_port #(
    .LINKWIDTH(LINKWIDTH),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  o_east(
    .clk(clk),
    .rst(rst),
    .write(out_port_write[`EAST_DIR]),
    .data_in(data_out_fifo_tail[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR]),
    .read(rdy_out[`EAST_DIR]),
    .data_out(data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR]),
    .empty(empty[`EAST_DIR]),
    .full(out_fifo_full[`EAST_DIR]));

output_port #(
    .LINKWIDTH(LINKWIDTH),
    .FIFO_ADDR_WIDTH(1),
    .FIFO_SIZE(2))
  o_local(
    .clk(clk),
    .rst(rst),
    .write(out_port_write[`LOCAL_DIR]),
    .data_in(data_out_fifo_tail[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR]),
    .read(rdy_out[`LOCAL_DIR]),
    .data_out(data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR]),
    .empty(empty[`LOCAL_DIR]),
    .full(out_fifo_full[`LOCAL_DIR]));

// ==================== INTERNAL LOGIC ====================

// connect requests between route computation and arbiters
assign req_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`NORTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`NORTH_DIR+`NORTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`WEST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`WEST_DIR+`NORTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`SOUTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`SOUTH_DIR+`NORTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`EAST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`EAST_DIR+`NORTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`LOCAL_DIR] = req_from_in_ports[OUT_PORT_CNT*`LOCAL_DIR+`NORTH_DIR];

assign req_to_out_ports[IN_PORT_CNT*`WEST_DIR+`NORTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`NORTH_DIR+`WEST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`WEST_DIR+`WEST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`WEST_DIR+`WEST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`WEST_DIR+`SOUTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`SOUTH_DIR+`WEST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`WEST_DIR+`EAST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`EAST_DIR+`WEST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`WEST_DIR+`LOCAL_DIR] = req_from_in_ports[OUT_PORT_CNT*`LOCAL_DIR+`WEST_DIR];

assign req_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`NORTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`NORTH_DIR+`SOUTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`WEST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`WEST_DIR+`SOUTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`SOUTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`SOUTH_DIR+`SOUTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`EAST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`EAST_DIR+`SOUTH_DIR];
assign req_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`LOCAL_DIR] = req_from_in_ports[OUT_PORT_CNT*`LOCAL_DIR+`SOUTH_DIR];

assign req_to_out_ports[IN_PORT_CNT*`EAST_DIR+`NORTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`NORTH_DIR+`EAST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`EAST_DIR+`WEST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`WEST_DIR+`EAST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`EAST_DIR+`SOUTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`SOUTH_DIR+`EAST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`EAST_DIR+`EAST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`EAST_DIR+`EAST_DIR];
assign req_to_out_ports[IN_PORT_CNT*`EAST_DIR+`LOCAL_DIR] = req_from_in_ports[OUT_PORT_CNT*`LOCAL_DIR+`EAST_DIR];

assign req_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`NORTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`NORTH_DIR+`LOCAL_DIR];
assign req_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`WEST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`WEST_DIR+`LOCAL_DIR];
assign req_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`SOUTH_DIR] = req_from_in_ports[OUT_PORT_CNT*`SOUTH_DIR+`LOCAL_DIR];
assign req_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`EAST_DIR] =  req_from_in_ports[OUT_PORT_CNT*`EAST_DIR+`LOCAL_DIR];
assign req_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`LOCAL_DIR] = req_from_in_ports[OUT_PORT_CNT*`LOCAL_DIR+`LOCAL_DIR];

// connect in_port_read with grt from arbiter
assign in_port_read[`NORTH_DIR] = grt_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`NORTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`WEST_DIR+`NORTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`NORTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`EAST_DIR+`NORTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`NORTH_DIR];

assign in_port_read[`WEST_DIR] = grt_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`WEST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`WEST_DIR+`WEST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`WEST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`EAST_DIR+`WEST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`WEST_DIR];

assign in_port_read[`SOUTH_DIR] = grt_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`SOUTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`WEST_DIR+`SOUTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`SOUTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`EAST_DIR+`SOUTH_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`SOUTH_DIR];

assign in_port_read[`EAST_DIR] = grt_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`EAST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`WEST_DIR+`EAST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`EAST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`EAST_DIR+`EAST_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`EAST_DIR];

assign in_port_read[`LOCAL_DIR] = grt_to_out_ports[IN_PORT_CNT*`NORTH_DIR+`LOCAL_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`WEST_DIR+`LOCAL_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`SOUTH_DIR+`LOCAL_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`EAST_DIR+`LOCAL_DIR] |
                                  grt_to_out_ports[IN_PORT_CNT*`LOCAL_DIR+`LOCAL_DIR];

// connect out_port_write with grt from arbiter
assign out_port_write[`NORTH_DIR] = |grt_to_out_ports[IN_PORT_CNT*(`NORTH_DIR+1)-1:IN_PORT_CNT*`NORTH_DIR];
assign out_port_write[`WEST_DIR] = |grt_to_out_ports[IN_PORT_CNT*(`WEST_DIR+1)-1:IN_PORT_CNT*`WEST_DIR];
assign out_port_write[`SOUTH_DIR] = |grt_to_out_ports[IN_PORT_CNT*(`SOUTH_DIR+1)-1:IN_PORT_CNT*`SOUTH_DIR];
assign out_port_write[`EAST_DIR] = |grt_to_out_ports[IN_PORT_CNT*(`EAST_DIR+1)-1:IN_PORT_CNT*`EAST_DIR];
assign out_port_write[`LOCAL_DIR] = |grt_to_out_ports[IN_PORT_CNT*(`LOCAL_DIR+1)-1:IN_PORT_CNT*`LOCAL_DIR];
assign rdy_in = ~full;
assign req_out = ~empty;

endmodule
